unit uMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  tapi, hbTapi, hbTapiUtils, StdCtrls, MMSystem, ExtCtrls,
  ComCtrls, Buttons, hbTapiWave;

type
  TForm1 = class(TForm)
    TapiLine: TTapiLine;
    WaveOut: TWaveOut;
    WaveIn: TWaveIn;
    ListView_Journal: TListView;
    Panel1: TPanel;
    ComboDeviceList: TComboBox;
    lbDevice: TLabel;
    StatusBar: TStatusBar;
    TimerToAnswer: TTimer;
    WaveOutSpeaker: TWaveOut;
    btnAction: TButton;
    cbAutoAnswer: TCheckBox;
    lbCaller: TLabel;
    edCall: TEdit;
    edFile: TEdit;
    lbFile: TLabel;
    SpeedButtonStop: TSpeedButton;
    SpeedButton1: TSpeedButton;
    OpenDialog1: TOpenDialog;
    SpeedButtonPlay: TSpeedButton;
    procedure ComboDeviceListChange(Sender: TObject);
    procedure WaveOutDone(Sender: TObject);
    procedure TapiLineConnected(Sender: TTapiLine; Call: TTapiCall);
    procedure TapiLineOffering(Sender: TTapiLine; Call: TTapiCall);
    procedure TapiLineIdle(Sender: TTapiLine; Call: TTapiCall);
    procedure TimerToAnswerTimer(Sender: TObject);
    procedure TapiLineCallState(Sender: TTapiLine; Call: TTapiCall;
      CallState: Cardinal);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ListView_JournalDblClick(Sender: TObject);
    procedure btnActionClick(Sender: TObject);
    procedure TapiLineCallerID(Sender: TTapiLine; Call: TTapiCall);
    procedure ComboDeviceListDropDown(Sender: TObject);
    procedure TapiLineDigit(Sender: TTapiLine; Call: TTapiCall;
      Digit: Char);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure SpeedButtonStopClick(Sender: TObject);
    procedure WaveOutSpeakerDone(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButtonPlayClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private-Deklarationen }
    procedure AddCallToList(Call : TTapiCall);
    function  GetCallerInfo(Call : TTapiCall) : String;
    procedure GetLineDevices;
    procedure Start(var Msg: TMessage); message WM_USER;
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.ComboDeviceListChange(Sender: TObject);
begin
  TapiLine.Active := False;
  if ComboDeviceList.ItemIndex >= 0 then begin
    TapiLine.Privileges.Owner := TRUE;
    TapiLine.DeviceID         := Integer(ComboDeviceList.Items.Objects[ComboDeviceList.ItemIndex]);
    TapiLine.Active           := True;
    btnAction.Enabled         := True;
  end;
end;

procedure TForm1.WaveOutDone(Sender: TObject);
var
  i : Integer;
  Filename : String;
begin
  WaveOut.Active := False;
  i := 0;
  repeat
    inc(i);
    Filename := Format('Rec%03.3d.wav', [i]);
  until not FileExists(Filename);
  try
    WaveIn.Filename := Filename;
    WaveIn.Active := True;
    StatusBar.Panels[1].Text := 'Recording';
  except
    On E:Exception do
      ShowMessage(E.Message);
  end;
end;

procedure TForm1.TapiLineConnected(Sender: TTapiLine; Call: TTapiCall);
begin
  if not Call.WaveOut.Available then begin
    ShowMessage('Could''t get wave/out handle');
    EXIT;
  end;

  if not Call.WaveIn.Available then begin
    ShowMessage('Could''t get wave/in handle');
    EXIT;
  end;

  WaveIn.DeviceID    := Call.WaveIn.DeviceID;
  WaveOut.DeviceID   := Call.WaveOut.DeviceID;
  WaveOut.PlayList.Clear;
  WaveOut.PlayList.Add(edFile.Text);
  WaveOut.PlayList.Add('Beep(1300,300)');
  WaveOut.Active     := True;
  StatusBar.Panels[1].Text := 'Playing...';
  btnAction.Caption := 'Drop';
// Feature of hbTapi Professional
// Call.MonitorDigits.Active := True;
end;

procedure TForm1.TapiLineOffering(Sender: TTapiLine; Call: TTapiCall);
begin
  if cbAutoAnswer.Checked then begin
    TimerToAnswer.Enabled := True;
    StatusBar.Panels[1].Text := Format('Ringing. Answer in %d sec.',[TimerToAnswer.Interval div 1000]);
  end;
  edCall.Text := GetCallerInfo(Call);
  btnAction.Caption := 'Answer';
end;

procedure TForm1.TapiLineIdle(Sender: TTapiLine; Call: TTapiCall);
begin
  AddCallToList(Call);
  WaveOut.Active   := False;
  WaveIn.Active := False;
  btnAction.Caption := 'Make Call';
  edCall.Text  := '';
  StatusBar.Panels[1].Text := 'Waiting...';
end;

procedure TForm1.TimerToAnswerTimer(Sender: TObject);
begin
  TimerToAnswer.Enabled := False;
  if TapiLine.Calls.Count > 0 then begin
    if (TapiLine.Calls[0].State = LINECALLSTATE_OFFERING) or
       (TapiLine.Calls[0].State = LINECALLSTATE_ACCEPTED) then
    begin
      TapiLine.Calls[0].Answer
    end
    else
      TapiLine.Calls[0].Drop;
  end;
end;

procedure TForm1.AddCallToList(Call: TTapiCall);
begin
  with ListView_Journal.Items.Add do
  begin
    Caption := DateTimetoStr(Call.BeginTime);
    SubItems.Add(GetCallerInfo(Call));
    if StatusBar.Panels[1].Text = 'Recording' then begin
      SubItems.Add(TimeToStr(WaveIn.Time));
      SubItems.Add(WaveIn.Filename);
    end;
  end;
end;

procedure TForm1.TapiLineCallState(Sender: TTapiLine; Call: TTapiCall;
  CallState: Cardinal);
begin
  StatusBar.Panels[0].Text := LineCallStateToStr(CallState);
  if (CallState = LINECALLSTATE_OFFERING) and
     (Call.Features and LINECALLFEATURE_ACCEPT > 0) then
  try
    Call.Accept;
  except
  end;
  TapiLineCallerID(Sender,Call);
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
var
  i : Integer;
begin
  for i := 0 to TapiLine.Calls.Count-1 do begin
    TapiLine.Calls[i].Drop;
  end;

  WaveOut.Active   := False;
  WaveIn.Active := False;
  TapiLine.Active   := False;
end;

procedure TForm1.ListView_JournalDblClick(Sender: TObject);
begin
  SpeedButtonPlayClick(nil);
end;

procedure TForm1.btnActionClick(Sender: TObject);
begin
  if TapiLine.Calls.Count = 0 then begin
    TapiLine.MakeCall(edCall.Text);
    btnAction.Caption := 'Drop';
  end else
    TimerToAnswerTimer(nil);
end;

procedure TForm1.TapiLineCallerID(Sender: TTapiLine; Call: TTapiCall);
begin
  edCall.Text := GetCallerInfo(Call);
end;

function TForm1.GetCallerInfo(Call: TTapiCall): String;
begin
  if Call.CallerID.Name <> '' then
    Call.Notes['Name']    := Call.CallerID.Name;
  if Call.CallerID.Address <> '' then
    Call.Notes['Address'] := Call.CallerID.Address;

  if Call.Notes['Name'] <> '' then
    result := Call.Notes['Name']+' - '+Call.Notes['Address']
  else
    result := Call.Notes['Address'];
end;

procedure TForm1.ComboDeviceListDropDown(Sender: TObject);
var
  i : Integer;
begin
  if ComboDeviceList.Items.Count > 0 then EXIT;
  // Search for possible tapi devices
  // normaly we need AUTOMATEDVOICE
  for i := 0 to TapiLine.DeviceList.count-1 do begin
    TapiLine.DeviceID := i;
    if (TapiLine.Caps.MediaModes and LINEMEDIAMODE_AUTOMATEDVOICE) <> 0 then
    try
      TapiLine.Active := True;
      if TapiLine.WaveIn.Available and TapiLine.WaveOut.Available then
        ComboDeviceList.Items.AddObject(
          Format('DeviceID: %02.2d - ',[i])+TapiLine.DeviceList[i],TObject(i));
    except
    end;
    TapiLine.Active := False;
  end;

  if ComboDeviceList.Items.Count = 0 then begin
    // if there is no AUTOMATEDVOICE device
    // we look INTERACTIVEVOICE

    for i := 0 to TapiLine.DeviceList.count-1 do begin
      TapiLine.DeviceID := i;
      if (TapiLine.Caps.MediaModes and LINEMEDIAMODE_INTERACTIVEVOICE) <> 0 then
      try
        TapiLine.Active := True;
        if TapiLine.WaveIn.Available and TapiLine.WaveOut.Available then
          ComboDeviceList.Items.AddObject(
            Format('DeviceID: %02.2d - ',[i])+TapiLine.DeviceList[i],TObject(i));
      except
      end;
      TapiLine.Active := False;
    end;
  end;
  if ComboDeviceList.Items.Count = 0 then
    ShowMessage('Could not find a TAPI device assigned with a wave device');

end;

procedure TForm1.GetLineDevices;
var
  i : Integer;
begin
  if ComboDeviceList.Items.Count > 0 then EXIT;
  TapiLine.Privileges.Owner := FALSE;
  // Search for possible tapi devices
  // normaly we need AUTOMATEDVOICE
  for i := 0 to TapiLine.DeviceList.count-1 do begin
    TapiLine.DeviceID := i;
    if (TapiLine.Caps.MediaModes and LINEMEDIAMODE_AUTOMATEDVOICE) <> 0 then
    try
      TapiLine.Active := True;
      if TapiLine.WaveIn.Available and TapiLine.WaveOut.Available then
        ComboDeviceList.Items.AddObject(
          Format('DeviceID: %02.2d - ',[i])+TapiLine.DeviceList[i],TObject(i));
    except
    end;
    TapiLine.Active := False;
  end;

  if ComboDeviceList.Items.Count = 0 then begin
    // if there is no AUTOMATEDVOICE device
    // we look INTERACTIVEVOICE

    for i := 0 to TapiLine.DeviceList.count-1 do begin
      TapiLine.DeviceID := i;
      if (TapiLine.Caps.MediaModes and LINEMEDIAMODE_INTERACTIVEVOICE) <> 0 then
      try
        TapiLine.Active := True;
        if TapiLine.WaveIn.Available and TapiLine.WaveOut.Available then
          ComboDeviceList.Items.AddObject(
            Format('DeviceID: %02.2d - ',[i])+TapiLine.DeviceList[i],TObject(i));
      except
      end;
      TapiLine.Active := False;
    end;
  end;
  if ComboDeviceList.Items.Count = 0 then
    ShowMessage('Could not find a TAPI device assigned with a wave device');

end;

// Feature of hbTapi Professional

procedure TForm1.TapiLineDigit(Sender: TTapiLine; Call: TTapiCall;
  Digit: Char);
begin
  case Digit of
    '*' :
    begin
      WaveOut.Active   := False;
      WaveOut.Active   := True;
      StatusBar.Panels[1].Text := 'restarted by *'
    end;
    '#' :
    begin
      WaveOutDone(nil);
    end;
  end;
end;

procedure TForm1.FormKeyPress(Sender: TObject; var Key: Char);
begin
  TapiLineDigit(nil,nil,Key);
end;

procedure TForm1.SpeedButtonStopClick(Sender: TObject);
begin
  WaveOutSpeaker.Active := False;
  SpeedButtonStop.Visible := False;
  SpeedButtonPlay.Visible := True;
end;

procedure TForm1.WaveOutSpeakerDone(Sender: TObject);
begin
  SpeedButtonStopClick(nil);
end;

procedure TForm1.SpeedButton1Click(Sender: TObject);
begin
  if OpenDialog1.Execute then begin
    edFile.Text := OpenDialog1.FileName;
    edFile.SetFocus;
  end;
end;

procedure TForm1.SpeedButtonPlayClick(Sender: TObject);
var
  aFilename : String;
begin
  if not WaveOutSpeaker.Active then begin
    aFilename := '';
    if ListView_Journal.Focused and (ListView_Journal.Items.Count>0) then begin
      if  assigned(ListView_Journal.Selected) and
         (ListView_Journal.Selected.SubItems.Count = 3)
      then
        aFilename := ListView_Journal.Selected.SubItems[2]
    end else
      aFilename := edFile.Text;
    if aFilename = '' then
      EXIT;
    WaveOutSpeaker.PlayList.Clear;
    WaveOutSpeaker.PlayList.Add(aFilename);
    WaveOutSpeaker.Active := True;
    SpeedButtonPlay.Visible := False;
    SpeedButtonStop.Visible := True;
  end;

end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  PostMessage(Handle, WM_USER, 0, 0);
end;

procedure TForm1.Start(var Msg: TMessage);
begin
  Update;
  Screen.Cursor := crHourGlass;
  try
    GetLineDevices;
  finally
    Screen.Cursor := crDefault;
  end;
end;

end.
